@@ -1,5 +1,5 @@
 /* Webcamoid, webcam capture application.
- * Copyright (C) 2011-2017  Gonzalo Exequiel Pedone
+ * Copyright (C) 2016  Gonzalo Exequiel Pedone
  *
  * Webcamoid is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,21 +20,29 @@
 #ifndef AUDIODEV_H
 #define AUDIODEV_H
 
-#include <akaudiopacket.h>
+#include <akaudiocaps.h>
 
-class AudioDeviceElement;
+class AudioDevPrivate;
+class AkAudioPacket;
 
 class AudioDev: public QObject
 {
     Q_OBJECT
+    Q_PROPERTY(int latency
+               READ latency
+               WRITE setLatency
+               RESET resetLatency
+               NOTIFY latencyChanged)
     Q_PROPERTY(QString error
                READ error
                NOTIFY errorChanged)
 
     public:
-        explicit AudioDev(QObject *parent=nullptr);
+        AudioDev(QObject *parent=nullptr);
         virtual ~AudioDev();
 
+        Q_INVOKABLE int latency() const;
+        Q_INVOKABLE const QVector<int> &commonSampleRates() const;
         Q_INVOKABLE virtual QString error() const;
         Q_INVOKABLE virtual QString defaultInput();
         Q_INVOKABLE virtual QString defaultOutput();
@@ -43,24 +51,27 @@ class AudioDev: public QObject
         Q_INVOKABLE virtual QString description(const QString &device);
         Q_INVOKABLE virtual AkAudioCaps preferredFormat(const QString &device);
         Q_INVOKABLE virtual QList<AkAudioCaps::SampleFormat> supportedFormats(const QString &device);
-        Q_INVOKABLE virtual QList<int> supportedChannels(const QString &device);
+        Q_INVOKABLE virtual QList<AkAudioCaps::ChannelLayout> supportedChannelLayouts(const QString &device);
         Q_INVOKABLE virtual QList<int> supportedSampleRates(const QString &device);
         Q_INVOKABLE virtual bool init(const QString &device, const AkAudioCaps &caps);
-        Q_INVOKABLE virtual QByteArray read(int samples);
+        Q_INVOKABLE virtual QByteArray read();
         Q_INVOKABLE virtual bool write(const AkAudioPacket &packet);
         Q_INVOKABLE virtual bool uninit();
 
-    protected:
-        QVector<int> m_commonSampleRates;
+    private:
+        AudioDevPrivate *d;
 
     Q_SIGNALS:
+        void latencyChanged(int latency);
         void errorChanged(const QString &error);
         void defaultInputChanged(const QString &defaultInput);
         void defaultOutputChanged(const QString &defaultOutput);
         void inputsChanged(const QStringList &inputs);
         void outputsChanged(const QStringList &outputs);
 
-    friend class AudioDeviceElement;
+    public Q_SLOTS:
+        void setLatency(int latency);
+        void resetLatency();
 };
 
 #endif // AUDIODEV_H
