@@ -1,5 +1,5 @@
 /* Webcamoid, webcam capture application.
- * Copyright (C) 2011-2017  Gonzalo Exequiel Pedone
+ * Copyright (C) 2016  Gonzalo Exequiel Pedone
  *
  * Webcamoid is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,25 +17,44 @@
  * Web-Site: http://webcamoid.github.io/
  */
 
+#include <QImage>
+#include <QReadWriteLock>
+#include <QVariant>
+#include <QQmlContext>
+#include <akpacket.h>
+#include <akpacket.h>
+#include <akvideopacket.h>
+
 #include "falsecolorelement.h"
 
+class FalseColorElementPrivate
+{
+    public:
+        QReadWriteLock m_mutex;
+        QList<QRgb> m_table {
+            qRgb(0, 0, 0),
+            qRgb(255, 0, 0),
+            qRgb(255, 255, 255),
+            qRgb(255, 255, 255)
+        };
+        bool m_soft {false};
+};
+
 FalseColorElement::FalseColorElement(): AkElement()
 {
-    this->m_table = {
-        qRgb(0, 0, 0),
-        qRgb(255, 0, 0),
-        qRgb(255, 255, 255),
-        qRgb(255, 255, 255)
-    };
+    this->d = new FalseColorElementPrivate;
+}
 
-    this->m_soft = false;
+FalseColorElement::~FalseColorElement()
+{
+    delete this->d;
 }
 
 QVariantList FalseColorElement::table() const
 {
     QVariantList table;
 
-    for (const QRgb &color: this->m_table)
+    for (auto &color: this->d->m_table)
         table << color;
 
     return table;
@@ -43,7 +62,16 @@ QVariantList FalseColorElement::table() const
 
 bool FalseColorElement::soft() const
 {
-    return this->m_soft;
+    return this->d->m_soft;
+}
+
+QRgb FalseColorElement::colorAt(int index)
+{
+    this->d->m_mutex.lockForRead();
+    auto color = this->d->m_table.at(index);
+    this->d->m_mutex.unlock();
+
+    return color;
 }
 
 QString FalseColorElement::controlInterfaceProvide(const QString &controlId) const
@@ -62,52 +90,20 @@ void FalseColorElement::controlInterfaceConfigure(QQmlContext *context,
     context->setContextProperty("controlId", this->objectName());
 }
 
-void FalseColorElement::setTable(const QVariantList &table)
+AkPacket FalseColorElement::iVideoStream(const AkVideoPacket &packet)
 {
-    QList<QRgb> tableRgb;
+    this->d->m_mutex.lockForRead();
+    auto isTableEmpty = this->d->m_table.isEmpty();
+    this->d->m_mutex.unlock();
 
-    for (const QVariant &color: table)
-        tableRgb << color.value<QRgb>();
+    if (isTableEmpty)  {
+        if (packet)
+            emit this->oStream(packet);
 
-    if (this->m_table == tableRgb)
-        return;
+        return packet;
+    }
 
-    this->m_table = tableRgb;
-    emit this->tableChanged(table);
-}
-
-void FalseColorElement::setSoft(bool soft)
-{
-    if (this->m_soft == soft)
-        return;
-
-    this->m_soft = soft;
-    emit this->softChanged(soft);
-}
-
-void FalseColorElement::resetTable()
-{
-    static const QVariantList table = {
-        qRgb(0, 0, 0),
-        qRgb(255, 0, 0),
-        qRgb(255, 255, 255),
-        qRgb(255, 255, 255)
-    };
-
-    this->setTable(table);
-}
-
-void FalseColorElement::resetSoft()
-{
-    this->setSoft(false);
-}
-
-AkPacket FalseColorElement::iStream(const AkPacket &packet)
-{
-    if (this->m_table.isEmpty())
-        akSend(packet)
-
-    QImage src = AkUtils::packetToImage(packet);
+    auto src = packet.toImage();
 
     if (src.isNull())
         return AkPacket();
@@ -116,12 +112,15 @@ AkPacket FalseColorElement::iStream(const AkPacket &packet)
     QImage oFrame(src.size(), QImage::Format_ARGB32);
 
     QRgb table[256];
-    QList<QRgb> tableRgb = this->m_table;
+
+    this->d->m_mutex.lockForRead();
+    auto tableRgb = this->d->m_table;
+    this->d->m_mutex.unlock();
 
     for (int i = 0; i < 256; i++) {
         QRgb color;
 
-        if (this->m_soft) {
+        if (this->d->m_soft) {
             int low = i * (tableRgb.size() - 1) / 255;
             low = qBound(0, low, tableRgb.size() - 2);
             int high = low + 1;
@@ -151,20 +150,141 @@ AkPacket FalseColorElement::iStream(const AkPacket &packet)
         } else {
             int t = tableRgb.size() * i / 255;
             t = qBound(0, t, tableRgb.size() - 1);
-            color = tableRgb[t];
+            int r = qRed(tableRgb[t]);
+            int g = qGreen(tableRgb[t]);
+            int b = qBlue(tableRgb[t]);
+            color = qRgb(r, g, b);
         }
 
         table[i] = color;
     }
 
     for (int y = 0; y < src.height(); y++) {
-        const quint8 *srcLine = src.constScanLine(y);
-        QRgb *dstLine = reinterpret_cast<QRgb *>(oFrame.scanLine(y));
+        auto srcLine = src.constScanLine(y);
+        auto dstLine = reinterpret_cast<QRgb *>(oFrame.scanLine(y));
 
         for (int x = 0; x < src.width(); x++)
             dstLine[x] = table[srcLine[x]];
     }
 
-    AkPacket oPacket = AkUtils::imageToPacket(oFrame, packet);
-    akSend(oPacket)
+    auto oPacket = AkVideoPacket::fromImage(oFrame, packet);
+
+    if (oPacket)
+        emit this->oStream(oPacket);
+
+    return oPacket;
 }
+
+void FalseColorElement::addColor(QRgb color)
+{
+    this->d->m_mutex.lockForWrite();
+    this->d->m_table << color;
+    this->d->m_mutex.unlock();
+
+    QVariantList table;
+
+    for (auto &color: this->d->m_table)
+        table << color;
+
+    emit this->tableChanged(table);
+}
+
+void FalseColorElement::setColor(int index, QRgb color)
+{
+    bool tableChanged = false;
+    this->d->m_mutex.lockForWrite();
+
+    if (index >= 0
+        && index < this->d->m_table.size()
+        && this->d->m_table[index] != color) {
+        this->d->m_table[index] = color;
+        tableChanged = true;
+    }
+
+    this->d->m_mutex.unlock();
+
+    if (tableChanged) {
+        QVariantList table;
+
+        for (auto &color: this->d->m_table)
+            table << color;
+
+        emit this->tableChanged(table);
+    }
+}
+
+void FalseColorElement::removeColor(int index)
+{
+    bool tableChanged = false;
+    this->d->m_mutex.lockForWrite();
+
+    if (index >= 0
+        && index < this->d->m_table.size()) {
+        this->d->m_table.removeAt(index);
+        tableChanged = true;
+    }
+
+    this->d->m_mutex.unlock();
+
+    if (tableChanged) {
+        QVariantList table;
+
+        for (auto &color: this->d->m_table)
+            table << color;
+
+        emit this->tableChanged(table);
+    }
+}
+
+void FalseColorElement::clearTable()
+{
+    this->setTable({});
+}
+
+void FalseColorElement::setTable(const QVariantList &table)
+{
+    QList<QRgb> tableRgb;
+
+    for (auto &color: table)
+        tableRgb << color.value<QRgb>();
+
+    this->d->m_mutex.lockForWrite();
+
+    if (this->d->m_table == tableRgb) {
+        this->d->m_mutex.unlock();
+
+        return;
+    }
+
+    this->d->m_table = tableRgb;
+    this->d->m_mutex.unlock();
+    emit this->tableChanged(table);
+}
+
+void FalseColorElement::setSoft(bool soft)
+{
+    if (this->d->m_soft == soft)
+        return;
+
+    this->d->m_soft = soft;
+    emit this->softChanged(soft);
+}
+
+void FalseColorElement::resetTable()
+{
+    static const QVariantList table = {
+        qRgb(0, 0, 0),
+        qRgb(255, 0, 0),
+        qRgb(255, 255, 255),
+        qRgb(255, 255, 255)
+    };
+
+    this->setTable(table);
+}
+
+void FalseColorElement::resetSoft()
+{
+    this->setSoft(false);
+}
+
+#include "moc_falsecolorelement.cpp"
