@@ -1,5 +1,5 @@
 /* Webcamoid, webcam capture application.
- * Copyright (C) 2011-2017  Gonzalo Exequiel Pedone
+ * Copyright (C) 2016  Gonzalo Exequiel Pedone
  *
  * Webcamoid is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,9 +20,10 @@
 #ifndef WAVEELEMENT_H
 #define WAVEELEMENT_H
 
-#include <QMutex>
-#include <ak.h>
-#include <akutils.h>
+#include <qrgb.h>
+#include <akelement.h>
+
+class WaveElementPrivate;
 
 class WaveElement: public AkElement
 {
@@ -49,7 +50,8 @@ class WaveElement: public AkElement
                NOTIFY backgroundChanged)
 
     public:
-        explicit WaveElement();
+        WaveElement();
+        ~WaveElement();
 
         Q_INVOKABLE qreal amplitude() const;
         Q_INVOKABLE qreal frequency() const;
@@ -57,18 +59,13 @@ class WaveElement: public AkElement
         Q_INVOKABLE QRgb background() const;
 
     private:
-        qreal m_amplitude;
-        qreal m_frequency;
-        qreal m_phase;
-        QRgb m_background;
-        QSize m_frameSize;
-        QVector<int> m_sineMap;
-        QMutex m_mutex;
+        WaveElementPrivate *d;
 
     protected:
         QString controlInterfaceProvide(const QString &controlId) const;
         void controlInterfaceConfigure(QQmlContext *context,
                                        const QString &controlId) const;
+        AkPacket iVideoStream(const AkVideoPacket &packet);
 
     signals:
         void amplitudeChanged(qreal amplitude);
@@ -86,7 +83,6 @@ class WaveElement: public AkElement
         void resetFrequency();
         void resetPhase();
         void resetBackground();
-        AkPacket iStream(const AkPacket &packet);
 
     private slots:
         void updateSineMap();
